﻿Module Module_MathConverter

    ' =====================================================================================
    '  KEYWORDS
    ' =====================================================================================
    Private VbNetOperatorsArray() As String
    Private VbNetKeywordsArray() As String
    Private MathKeywordsArray() As String

    ' ---------------------------------
    ' Symbol	 Aspect	   Unicode
    ' ---------------------------------
    ' hyphen	 – 	––––   002D or 2010
    ' minus sign −  −−−−   2212
    ' N-dash     –  ––––   2013
    ' M-dash     —  ————   2014

    Private SubtractionChars As String = ChrW(&H2D) + " " + _
                                         ChrW(&H2010) + " " + _
                                         ChrW(&H2212) + " " + _
                                         ChrW(&H2013) + " " + _
                                         ChrW(&H2014)

    Private SubtractionCharsUnspaced As String = RemoveAllSpacesAndTab(SubtractionChars)

    Private MathOperators As String = SubtractionCharsUnspaced + "*/+=≈^±!"

    Friend Sub MathConverter_Init()
        Dim VbNetOperators As String = SubtractionChars + " ( ) * / + = ^"
        VbNetOperatorsArray = VbNetOperators.Split(" "c)

        Dim VbNetKeywords As String = " Math.E Math.PI"
        VbNetKeywords += " Acos Acosh Asin Asinh Atan Atanh"
        VbNetKeywords += " Cos Cosh Exp Log Log2 Log10"
        VbNetKeywords += " Sin Sinh Sqrt Tan Tanh"
        VbNetKeywords += " Abs Pow"
        VbNetKeywords += " Math.Acos Math.Acosh Math.Asin Math.Asinh Math.Atan Math.Atanh"
        VbNetKeywords += " Math.Cos Math.Cosh Math.Exp Math.Log Math.Log2 Math.Log10"
        VbNetKeywords += " Math.Sin Math.Sinh Math.Sqrt Math.Tan Math.Tanh"
        VbNetKeywords += " Math.Abs Math.Pow"
        VbNetKeywordsArray = VbNetKeywords.Split(" "c)
        GnomeSort_ForLength(VbNetKeywordsArray)
        '
        ' TODO
        'Dim MathKeywords As String = SubtractionChars + " arcsin log ln ( ) * / + = ≈ ^ " ' old
        Dim MathKeywords As String = SubtractionChars + " ( ) * / + = ≈ ^ " ' new
        MathKeywordsArray = MathKeywords.Split(" "c)
        GnomeSort_ForLength(MathKeywordsArray)
    End Sub



    ' =====================================================================================
    '  IDENTIFIERS
    ' =====================================================================================
    Structure Identifier
        Dim LongName As String
        Dim Letter As String
    End Structure
    Private Identifiers(-1) As Identifier

    Private Function GetVarLettersFromIdentifiers() As String
        Dim s As String = ""
        For Each i As Identifier In Identifiers
            s += " " + i.Letter
        Next
        Return s.Trim
    End Function

    Friend Sub AddIdentifiersToListView(ByRef lv As ListView)
        lv.Items.Clear()
        For i As Int32 = 0 To Identifiers.Length - 1
            With Identifiers(i)
                lv.Items.Add("     " + .Letter)
                lv.Items(i).SubItems.Add(.LongName)
            End With
        Next
    End Sub

    Friend Sub SetLetterToIdentifierArray(ByVal i As Int32, ByVal s As String)
        Identifiers(i).Letter = s
    End Sub

    Friend Function GetIdentifiersAsString() As String
        Dim s As String = ""
        For i As Int32 = 0 To Identifiers.Length - 1
            With Identifiers(i)
                s += "Identifier= " + .Letter + " " + .LongName + vbCrLf
            End With
        Next
        Return s
    End Function

    Friend Sub AddIdentifierFromString(ByVal s As String)
        s = s.Trim
        Dim sa As String() = s.Split(" "c)
        ReDim Preserve Identifiers(Identifiers.Length)
        Identifiers(Identifiers.Length - 1).Letter = sa(0)
        Identifiers(Identifiers.Length - 1).LongName = sa(1)
    End Sub

    Private Function IdentifierSingleLetterExists(ByVal s As String) As Boolean
        For Each id As Identifier In Identifiers
            If id.Letter IsNot Nothing Then
                If s.ToLower = id.Letter.ToLower Then Return True
            End If
        Next
        Return False
    End Function

    Private Function IdentifiersLongNameExists(ByVal s As String) As Boolean
        For Each id As Identifier In Identifiers
            If s = id.LongName Then Return True
        Next
        Return False
    End Function

    Friend Function GetIdentifierSingleLetterIndex(ByVal str As String) As Int32
        For i As Int32 = 0 To Identifiers.Length - 1
            If str.ToLower = Identifiers(i).Letter.ToLower Then Return i
        Next
        Return -1
    End Function

    Friend Function GetIdentifierLongNameIndex(ByVal str As String) As Int32
        For i As Int32 = 0 To Identifiers.Length - 1
            If str.ToLower = Identifiers(i).LongName.ToLower Then Return i
        Next
        Return -1
    End Function

    Friend Function GetIdentifierSingleLetter_LowerCaseOrUpperCase(ByVal str As String) As String
        For i As Int32 = 0 To Identifiers.Length - 1
            If str.ToLower = Identifiers(i).Letter.ToLower Then Return Identifiers(i).Letter
        Next
        Return ""
    End Function


    ' =====================================================================================
    '  PARSE IDENTIFIERS
    ' =====================================================================================
    Friend Sub ParseIdentifiers(ByVal s As String)
        If s = "" Then
            ReDim Identifiers(-1)
            Return
        End If
        ' --------------------------------------- isolate vars
        s = SpacesAroundOperators(s)
        s = RemoveKeywords(s)
        s = ReplaceMultipleSpacesAndTabAndTrim(s)
        Dim sa() As String = ExtractVars(s)
        GnomeSort_ForLength(sa)
        ' --------------------------------------- create identifiers
        AssignVarsToIdentifiers(sa)
        AssignSingleLettersToIdentifiers()
    End Sub

    Private Function SpacesAroundOperators(ByVal s As String) As String
        For Each Str As String In VbNetOperatorsArray
            s = s.Replace(Str, " " + Str + " ")
        Next
        Return s
    End Function

    Private Function RemoveKeywords(ByVal s As String) As String
        For Each Str As String In VbNetKeywordsArray
            s = Strings.Replace(s, " " + Str + "(", " ", , , CompareMethod.Text)
            s = Strings.Replace(s, " " + Str + " ", " ", , , CompareMethod.Text)
            s = Strings.Replace(s, " " + Str.ToLower + "(", " ", , , CompareMethod.Text)
            s = Strings.Replace(s, " " + Str.ToLower + " ", " ", , , CompareMethod.Text)
            s = Strings.Replace(s, " " + "Math." + Str + "(", " ", , , CompareMethod.Text)
            s = Strings.Replace(s, " " + "Math." + Str + " ", " ", , , CompareMethod.Text)
            s = Strings.Replace(s, " " + "Math." + Str.ToLower + "(", " ", , , CompareMethod.Text)
            s = Strings.Replace(s, " " + "Math." + Str.ToLower + " ", " ", , , CompareMethod.Text)
        Next
        For Each Str As String In VbNetOperatorsArray
            s = Strings.Replace(s, Str, " ")
        Next
        Return s.Trim
    End Function

    Private Function ExtractVars(ByVal str As String) As String()
        Dim sa As String() = str.Split(" "c)
        Dim sb(-1) As String
        For Each s As String In sa
            If Not IsNumeric(s) Then
                If Not ArrayContainsString(sb, s) Then
                    ReDim Preserve sb(sb.Length)
                    sb(sb.Length - 1) = s
                End If
            End If
        Next
        Return sb
    End Function

    Private Sub GnomeSort_ForLength(ByRef ar() As String)
        Dim pos As Int32 = 1
        While pos < ar.Length
            If ar(pos).Length <= ar(pos - 1).Length Then
                pos += 1
            Else
                Swap(ar(pos), ar(pos - 1))
                If pos > 1 Then pos -= 1
            End If
        End While
    End Sub

    Private Sub Swap(Of T)(ByRef a As T, ByRef b As T)
        Dim temp As T
        temp = a
        a = b
        b = temp
    End Sub

    Private Sub AssignVarsToIdentifiers(ByVal vars() As String)
        ReDim Identifiers(-1)
        For Each s As String In vars
            ReDim Preserve Identifiers(Identifiers.Length)
            Identifiers(Identifiers.Length - 1).LongName = s
        Next
    End Sub

    Private Sub AssignSingleLettersToIdentifiers()
        Dim letters() As String = {"a", "b", "c", "d", "f", "g", "h", "j", "k", "l", "m", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"}
        Dim index As Int32 = 0
        Dim l As Int32 = 0
        For i As Int32 = Identifiers.Length - 1 To 0 Step -1
            Dim s As String = Identifiers(i).LongName.Substring(0, 1).ToLower
            Do
                If Not IdentifierSingleLetterExists(s) And letters.Contains(s) Then Exit Do
                s = letters(l)
                l += 1
            Loop
            If s <> "x" And s <> "y" And s <> "z" Then s = s.ToUpper
            Identifiers(i).Letter = s
        Next
    End Sub


    ' =====================================================================================
    '  TRANSLATE TO MATH NOTATION
    ' =====================================================================================
    Friend Function TranslateToMathNotation(ByVal s As String) As String
        s = MathNotation_SetRegularSpacing_ForKeywords(s)
        s = ReplaceLongVarsWithLetters(s)
        s = EnsureCleanMathNotation(s)
        s = MathNotation_SetRegularSpacing_ForKeywords(s)
        Return s
    End Function

    Friend Function ReplaceLongVarsWithLetters(ByVal s As String) As String
        s = " " + s + " "
        For Each id As Identifier In Identifiers
            s = s.Replace(" " + id.LongName + " ", " " + id.Letter + " ")
        Next
        Return s
    End Function

    Friend Function EnsureCleanMathNotation(ByVal s As String) As String
        s = Strings.Replace(s, "Exp", "e ^ ", , , CompareMethod.Text)
        s = Strings.Replace(s, "Math.E", "e", , , CompareMethod.Text)
        s = Strings.Replace(s, "Math.PI", "π", , , CompareMethod.Text)
        s = Strings.Replace(s, "Math.", "", , , CompareMethod.Text)
        s = Strings.Replace(s, "lbase2", "Log2", , , CompareMethod.Text)
        s = Strings.Replace(s, "lbase10", "Log10", , , CompareMethod.Text)
        Return s
    End Function

    ' =====================================================================================
    '  TRANSLATE TO Dot-Net NOTATION
    ' =====================================================================================
    Friend Function TranslateToDotNetNotation(ByVal str As String) As String
        ' ------------------------------------------------- remove the "and" and "for" conditions
        str = RemoveTheAndConditions(str)
        str = RemoveTheForConditions(str)
        ' ------------------------------------------------- split the text to lines
        Dim sa As String() = SplitTextToLines(str)
        ' ------------------------------------------------- process all the lines
        str = ""
        For Each s As String In sa
            If Mid(s, 2, 3) <> " = " And Mid(s, 2, 1) <> "≈" Then Continue For
            If s.Contains("Wolfram") Then Continue For
            If s.StartsWith("No response") Then Continue For
            If s.StartsWith("Timed out pods") Then Continue For
            If s = "" Then Continue For
            s = MathNotation_SetRegularSpacing_ForKeywords(s)
            s = ReplaceLettersWithLongVars(s)
            s = EnsureCleanVbNetNotation(s)
            s = AddMultiplySymbols(s)
            str += s + vbCrLf
        Next
        str = Remove_2xPIxN(str)
        str = RemoveDuplicatedLines(str)
        ' ------------------------------------------------- remove the final CrLf and return
        Return RemoveTheFinalCrLf(str)
    End Function

    Friend Function MathNotation_SetRegularSpacing_ForKeywords(ByVal s As String) As String
        ' -----------------------------------------------------------------------------
        For Each Str As String In MathKeywordsArray
            s = Strings.Replace(s, Str, " " + Str + " ", 1, -1, CompareMethod.Text)
        Next
        ' -----------------------------------------------------------------------------
        s = ReplaceMultipleSpacesAndTabAndTrim(s)
        Return s
    End Function

    Friend Function ReplaceLettersWithLongVars(ByVal s As String) As String
        s = " " + s + " "
        For Each id As Identifier In Identifiers
            s = s.Replace(" " + id.Letter + " ", " " + id.LongName + " ")
        Next
        Return s.Trim
    End Function

    ' --------------------------------------------------------------------------------------------------
    ' DERIVED MATH FUNCTIONS
    ' https://github.com/nemrism/dotnet-docs/blob/master/docs/visual-basic/language-reference/keywords/derived-math-functions.md
    ' --------------------------------------------------------------------------------------------------

    Friend Function EnsureCleanVbNetNotation(ByVal s As String) As String

        s = Strings.Replace(s, " arcsin ", " Asin ", , , CompareMethod.Text)
        s = Strings.Replace(s, " arccos ", " Acos ", , , CompareMethod.Text)
        s = Strings.Replace(s, " arctan ", " Atan ", , , CompareMethod.Text)

        s = Strings.Replace(s, " csc ( ", " 1 / Sin ( ", , , CompareMethod.Text)
        s = Strings.Replace(s, " sec ( ", " 1 / Cos ( ", , , CompareMethod.Text)
        s = Strings.Replace(s, " cot ( ", " 1 / Tan ( ", , , CompareMethod.Text)

        ' --------------------------------------------------------------------------
        ' It is not possible to write the "csc" and "sec" equivalents in plain text
        ' --------------------------------------------------------------------------
        ' If you receive a "csc ^ ( - 1 )" replace it with the following function: 
        ' Function Arccosec(ByVal x As Double) as Double
        ' {
        '   Return Math.Atan(Math.Sign(x) / Math.Sqrt(x * x - 1))
        ' }
        ' --------------------------------------------------------------------------
        ' If you receive a "sec ^ ( - 1 )" replace it with the following function:
        ' Function Arcsec(ByVal x As Double) as Double
        ' {
        '   Return 2 * Math.Atan(1) - Math.Atan(Math.Sign(x) / Math.Sqrt(x * x - 1))
        ' }
        ' --------------------------------------------------------------------------

        s = Strings.Replace(s, " cot ^ ( - 1 ) ( ", " Math.PI / 2 - Math.Atan ( ", , , CompareMethod.Text)

        s = Strings.Replace(s, " sin ( ", " Math.Sin ( ", , , CompareMethod.Text)
        s = Strings.Replace(s, " cos ( ", " Math.Cos ( ", , , CompareMethod.Text)
        s = Strings.Replace(s, " tan ( ", " Math.Tan ( ", , , CompareMethod.Text)

        s = Strings.Replace(s, " sin ^ ( - 1 ) ( ", " Math.Asin ( ", , , CompareMethod.Text)
        s = Strings.Replace(s, " cos ^ ( - 1 ) ( ", " Math.Acos ( ", , , CompareMethod.Text)
        s = Strings.Replace(s, " tan ^ ( - 1 ) ( ", " Math.Atan ( ", , , CompareMethod.Text)

        s = Strings.Replace(s, " sqrt ( ", " Math.Sqrt ( ", , , CompareMethod.Text)

        s = Strings.Replace(s, " log2 ( ", " 1 / Math.Log(2) * Math.Log ( ", , , CompareMethod.Text)
        s = Strings.Replace(s, " log10 ( ", " Math.Log10 ( ", , , CompareMethod.Text)
        s = Strings.Replace(s, " log ( ", " Math.Log ( ", , , CompareMethod.Text)

        s = Strings.Replace(s, " exp ( ", " Math.E ^ ( ", , , CompareMethod.Text)

        s = Strings.Replace(s, " e ", " Math.E ", , , CompareMethod.Text)
        s = Strings.Replace(s, " π ", " Math.PI ")

        s = Strings.Replace(s, " ≈ ", " = ")
        Return s
    End Function

    Friend Function AddMultiplySymbols(ByVal s As String) As String
        Dim sa As String() = s.Split(" "c)
        Dim types As String() = DecodeTypes(sa)
        Dim s2 As String = ""
        ' ------------------------------------------------------ test each couple
        For i As Int32 = 0 To sa.Length - 1
            s2 += sa(i) + " "
            If i < sa.Length - 1 Then
                ' ---------------------------------------------- left and right types
                Dim leftType As String = types(i)
                Dim rightType As String = types(i + 1)
                ' ---------------------------------------------- 
                If leftType = "Operator" Then Continue For
                If rightType = "Operator" Then Continue For
                If leftType = "Keyword" Then Continue For
                If leftType = "Open" Then Continue For
                If rightType = "Close" Then Continue For
                s2 += "* "
            End If
        Next
        Return s2.Trim
    End Function

    Private Function DecodeTypes(ByVal sa As String()) As String()
        Dim sb(sa.Length - 1) As String
        For i As Int32 = 0 To sa.Length - 1
            sb(i) = "Unknown term"
            If ArrayContainsString(VbNetKeywordsArray, sa(i)) Then sb(i) = "Keyword"
            If sa(i) = "(" Then sb(i) = "Open" : Continue For
            If sa(i) = ")" Then sb(i) = "Close" : Continue For
            If sa(i) = "Math.E" Then sb(i) = "Const" : Continue For
            If sa(i) = "Math.PI" Then sb(i) = "Const" : Continue For
            If MathOperators.Contains(sa(i)) Then sb(i) = "Operator" : Continue For
            If IdentifiersLongNameExists(sa(i)) Then sb(i) = "Identifier"
            If IsNumeric(sa(i)) Then sb(i) = "Number"
        Next
        Return sb
    End Function

    Friend Function RemoveTheAndConditions(ByVal s As String) As String
        s = s.Replace(Chr(10), "")
        Dim sa As String() = s.Split(Chr(13))
        s = ""
        For Each Str As String In sa
            Dim i As Int32 = Str.IndexOf("and")
            If i > 0 Then Str = Str.Remove(i).TrimEnd
            If i = 0 Then Str = ""
            Str = Str.Trim
            If Str <> "" Then
                s += Str + vbCrLf
            End If
        Next
        Return s
    End Function

    Friend Function RemoveTheForConditions(ByVal s As String) As String
        s = s.Replace(Chr(10), "")
        Dim sa As String() = s.Split(Chr(13))
        s = ""
        For Each Str As String In sa
            Dim i As Int32 = Str.IndexOf("for")
            If i > 0 Then Str = Str.Remove(i).TrimEnd
            If i = 0 Then Str = ""
            Str = Str.Trim
            If Str <> "" Then
                s += Str + vbCrLf
            End If
        Next
        Return s
    End Function

    Private Function Remove_2xPIxN(ByVal str As String) As String
        str = str.Replace(" + 2 * Math.PI * n + π", "")
        str = str.Replace(" + 2 * Math.PI * n", "")
        str = str.Replace(" 2 * Math.PI * n + π", "")
        str = str.Replace(" 2 * Math.PI * n", "")
        Return (str)
    End Function

    Private Function RemoveDuplicatedLines(ByVal str As String) As String
        str = str.Replace(vbCrLf, vbLf)
        Dim sa As String() = str.Split(Chr(10))
        For i As Int32 = sa.Length - 1 To 1 Step -1
            For j As Int32 = 0 To i - 1
                If sa(i) = sa(j) Then
                    sa(i) = ""
                End If
            Next
        Next
        str = ""
        For i As Int32 = 0 To sa.Length - 1
            If sa(i) <> "" Then
                str += sa(i) + vbLf
            End If
        Next
        str = str.Replace(vbLf, vbCrLf)
        Return (str)
    End Function

    Friend Function RemoveTheFinalCrLf(ByVal s As String) As String
        Dim i As Int32 = s.LastIndexOf(vbCrLf)
        If i > 0 Then s = s.Remove(i).TrimEnd
        Return s
    End Function

End Module
