﻿Imports System.Runtime.CompilerServices

Module Module_Utils

    ' ==============================================================
    '  STRING HELPERS
    ' ==============================================================
    Friend Function ReplaceMultipleSpacesAndTabAndTrim(ByVal s As String) As String
        If s = "" Then Return s
        s = s.Replace(vbTab, " ")
        While s.Contains("  ")
            s = s.Replace("  ", " ")
        End While
        Return s.Trim
    End Function

    Friend Function RemoveAllSpacesAndTab(ByVal s As String) As String
        If s = "" Then Return s
        s = s.Replace(vbTab, " ")
        While s.Contains(" ")
            s = s.Replace(" ", "")
        End While
        Return s
    End Function

    Friend Function ArrayContainsString(ByVal sa As String(), ByVal s As String) As Boolean
        For Each str As String In sa
            If s = str Then Return True
        Next
        Return False
    End Function

    Friend Function SplitTextToLines(ByVal s As String) As String()
        s = s.Replace(Chr(10), "")
        Return s.Split(Chr(13))
    End Function

    ' =============================================================================
    '  RICH TEXT BOX - EXTENSIONS - GetFirstSelectedLine
    ' =============================================================================
    <Extension()> _
    Friend Function GetFirstSelectedLine(ByVal rtb As RichTextBox) As String
        rtb.WordWrap = False
        Dim i As Int32 = rtb.GetLineFromCharIndex(rtb.SelectionStart)
        rtb.WordWrap = True
        If i >= 0 Then
            Return rtb.Lines(i)
        Else
            Return ""
        End If
    End Function

    ' =============================================================================
    '  RICH TEXT BOX - EXTENSIONS - GetFirstSelectedLineIndex
    ' =============================================================================
    <Extension()> _
    Friend Function GetFirstSelectedLineIndex(ByVal rtb As RichTextBox) As Int32
        rtb.WordWrap = False
        GetFirstSelectedLineIndex = rtb.GetLineFromCharIndex(rtb.SelectionStart)
        rtb.WordWrap = True
    End Function

    ' =============================================================================
    '  RICH TEXT BOX - EXTENSIONS - SelectLine
    ' =============================================================================
    <Extension()> _
    Friend Sub SelectLine(ByVal rtb As RichTextBox, ByVal line As Int32)
        rtb.WordWrap = False
        If line >= 0 AndAlso line < rtb.Lines.Length AndAlso rtb.Lines.Length > 0 Then
            Dim lineStart As Int32 = rtb.GetFirstCharIndexFromLine(line)
            Dim lineLength As Int32 = rtb.Lines(line).Length
            rtb.Select(lineStart, lineLength)
        Else
            rtb.Select(0, 0)
        End If
        rtb.WordWrap = True
    End Sub


    ' =============================================================================
    '  RICH TEXT BOX - EXTENSION - SUPPRESS SOUNDS and correct CTRL-Keys and Undo
    ' =============================================================================
    <Extension()> _
    Friend Sub SuppressSounds_AndCorrect_CtrlKeys_and_Undo(ByVal rtb As RichTextBox)
        AddHandler rtb.KeyDown, AddressOf RTB_KeyDown
    End Sub
    Private Sub RTB_KeyDown(ByVal sender As Object, ByVal e As KeyEventArgs)
        '
        Dim rtb As RichTextBox = CType(sender, RichTextBox)
        ' -------------------------------------------- Suppress sounds
        If rtb.GetLineFromCharIndex(rtb.SelectionStart) = 0 AndAlso e.KeyData = Keys.Up _
        OrElse rtb.GetLineFromCharIndex(rtb.SelectionStart) = rtb.GetLineFromCharIndex(rtb.TextLength) AndAlso e.KeyData = Keys.Down _
        OrElse rtb.SelectionStart = rtb.TextLength AndAlso e.KeyData = Keys.Right _
        OrElse rtb.SelectionStart = 0 AndAlso _
                (e.KeyData = Keys.Left Or (e.KeyData = Keys.Back And rtb.SelectionLength = 0)) Then
            e.Handled = True
        End If
        '
        If e.Control Then
            Select Case e.KeyCode
                ' ------------------------------------ Suppress CTRL-E/R/L
                Case Keys.E, Keys.R, Keys.L
                    e.Handled = True
                    ' -------------------------------- Keep working the Undo buffer when pasting text with CTRL-V 
                Case Keys.V
                    rtb.SelectedText = My.Computer.Clipboard.GetText
                    e.Handled = True
                    ' -------------------------------- Remove the blue selection when doing an Undo with CTRL-Z
                Case Keys.Z
                    rtb.Undo()
                    rtb.SelectionLength = 0
                    e.Handled = True
                Case Keys.V
            End Select
        End If
        '
        'KeyEventArgs_DebugMessage(e)
    End Sub

    ' ==============================================================
    '  KeyEventArgs Debug Message
    ' ==============================================================
    Private Sub KeyEventArgs_DebugMessage(ByVal e As KeyEventArgs)
        Dim str As String = ""
        str += "Alt = " + e.Alt.ToString() + vbCrLf
        str += "Control = " + e.Control.ToString() + vbCrLf
        str += "Handled = " + e.Handled.ToString() + vbCrLf
        str += "KeyCode = " + e.KeyCode.ToString() + vbCrLf
        str += "KeyValue = " + e.KeyValue.ToString() + vbCrLf
        str += "KeyData = " + e.KeyData.ToString() + vbCrLf
        str += "Modifiers = " + e.Modifiers.ToString() + vbCrLf
        str += "Shift = " + e.Shift.ToString() + vbCrLf
        str += "SuppressKeyPress = " + e.SuppressKeyPress.ToString() + vbCrLf
        MessageBox.Show(str, "KeyEventArgs")
    End Sub

    ' ==============================================================
    '  RICH TEXT BOX - EXTENSION - GetTextWithCRLF 
    ' ==============================================================
    '  When setting the RichTextBox.Text, you could use LF or CRLF
    '  When reading the RichTextBox.Text, you get the LF only
    ' --------------------------------------------------------------
    <Extension()> _
    Friend Function GetTextWithCRLF(ByVal rtb As RichTextBox) As String
        Dim s As String = ""
        s = rtb.Text
        s = s.Replace(vbCr, "")
        s = s.Replace(vbLf, vbCrLf)
        Return s
    End Function

    ' ==============================================================
    '  RICH TEXT BOX - EXTENSION - RIGHT BUTTON MENU 
    ' ==============================================================
    <Extension()> _
    Friend Sub AddContextMenu(ByVal rtb As RichTextBox)
        If rtb.ContextMenuStrip Is Nothing Then
            '
            Dim tsmiUndo As ToolStripMenuItem = New ToolStripMenuItem("Undo")
            Dim tsmiRedo As ToolStripMenuItem = New ToolStripMenuItem("Redo")
            Dim tsmiCut As ToolStripMenuItem = New ToolStripMenuItem("Cut")
            Dim tsmiCopy As ToolStripMenuItem = New ToolStripMenuItem("Copy")
            Dim tsmiPaste As ToolStripMenuItem = New ToolStripMenuItem("Paste")
            Dim tsmiDelete As ToolStripMenuItem = New ToolStripMenuItem("Delete")
            Dim tsmiSelectAll As ToolStripMenuItem = New ToolStripMenuItem("Select All")
            '
            tsmiUndo.Tag = rtb
            tsmiRedo.Tag = rtb
            tsmiCut.Tag = rtb
            tsmiCopy.Tag = rtb
            tsmiPaste.Tag = rtb
            tsmiDelete.Tag = rtb
            tsmiSelectAll.Tag = rtb
            '
            Dim cms As ContextMenuStrip = New ContextMenuStrip()
            cms.ShowImageMargin = False
            cms.Font = New Font("Tahoma", 11)
            rtb.ContextMenuStrip = cms
            cms.Tag = rtb
            '
            AddHandler tsmiUndo.Click, AddressOf RTB_Undo
            AddHandler tsmiRedo.Click, AddressOf RTB_Redo
            AddHandler tsmiCut.Click, AddressOf RTB_Cut
            AddHandler tsmiCopy.Click, AddressOf RTB_Copy
            AddHandler tsmiPaste.Click, AddressOf RTB_Paste
            AddHandler tsmiDelete.Click, AddressOf RTB_Delete
            AddHandler tsmiSelectAll.Click, AddressOf RTB_SelectAll
            AddHandler cms.Opening, AddressOf CMS_Opening
            '
            cms.Items.Add(tsmiUndo)
            cms.Items.Add(tsmiRedo)
            cms.Items.Add(New ToolStripSeparator())
            cms.Items.Add(tsmiCut)
            cms.Items.Add(tsmiCopy)
            cms.Items.Add(tsmiPaste)
            cms.Items.Add(tsmiDelete)
            cms.Items.Add(New ToolStripSeparator())
            cms.Items.Add(tsmiSelectAll)
        End If
    End Sub
    Private Sub RTB_Undo(ByVal sender As Object, ByVal e As System.EventArgs)
        CType(CType(sender, ToolStripMenuItem).Tag, RichTextBox).Undo()
        CType(CType(sender, ToolStripMenuItem).Tag, RichTextBox).SelectionLength = 0
    End Sub
    Private Sub RTB_Redo(ByVal sender As Object, ByVal e As System.EventArgs)
        CType(CType(sender, ToolStripMenuItem).Tag, RichTextBox).Redo()
    End Sub
    Private Sub RTB_Cut(ByVal sender As Object, ByVal e As System.EventArgs)
        CType(CType(sender, ToolStripMenuItem).Tag, RichTextBox).Cut()
    End Sub
    Private Sub RTB_Copy(ByVal sender As Object, ByVal e As System.EventArgs)
        CType(CType(sender, ToolStripMenuItem).Tag, RichTextBox).Copy()
    End Sub
    Private Sub RTB_Paste(ByVal sender As Object, ByVal e As System.EventArgs)
        CType(CType(sender, ToolStripMenuItem).Tag, RichTextBox).Paste()
    End Sub
    Private Sub RTB_Delete(ByVal sender As Object, ByVal e As System.EventArgs)
        CType(CType(sender, ToolStripMenuItem).Tag, RichTextBox).SelectedText = ""
    End Sub
    Private Sub RTB_SelectAll(ByVal sender As Object, ByVal e As System.EventArgs)
        CType(CType(sender, ToolStripMenuItem).Tag, RichTextBox).SelectAll()
    End Sub
    Private Sub CMS_Opening(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim cms As ContextMenuStrip = CType(sender, ContextMenuStrip)
        Dim rtb As RichTextBox = CType(cms.Tag, RichTextBox)
        cms.Items(0).Enabled = Not rtb.[ReadOnly] AndAlso rtb.CanUndo
        cms.Items(1).Enabled = Not rtb.[ReadOnly] AndAlso rtb.CanRedo
        cms.Items(3).Enabled = Not rtb.[ReadOnly] AndAlso rtb.SelectionLength > 0
        cms.Items(4).Enabled = rtb.SelectionLength > 0
        cms.Items(5).Enabled = Not rtb.[ReadOnly] AndAlso Clipboard.ContainsText()
        cms.Items(6).Enabled = Not rtb.[ReadOnly] AndAlso rtb.SelectionLength > 0
        cms.Items(8).Enabled = rtb.TextLength > 0 AndAlso rtb.SelectionLength < rtb.TextLength
    End Sub


    ' ===================================================================================
    '  RICH TEXT BOX - EXTENSION - Create an undo buffer entry, assigning to selected text
    ' ===================================================================================
    <Extension()> _
    Friend Sub SetTextUsingUndo(ByVal rtb As RichTextBox, ByVal s As String)
        rtb.SelectAll()
        rtb.SelectedText = s
        rtb.SelectionStart = 0
        rtb.SelectionLength = 0
        rtb.Select()
    End Sub

End Module
