﻿
Module Module_Wolfram

    Friend AppId As String = "GA4T4H-LHP4GW9EL7"

    Private QueryString As String

    Friend Sub Wolfram_CreateQuery(ByVal str As String, ByVal querytype As String, ByVal p As Point)
        querytype = querytype.ToLower
        If querytype = "solve" Then
            QueryString = ""
            str = str + " "
            If str.ToLower.StartsWith("solve for ") Then
                QueryString = str
            ElseIf str.ToLower.StartsWith("x =") And str.Contains(" y ") Then
                QueryString = "solve for y " + str
            ElseIf str.ToLower.StartsWith("x =") And str.Contains(" Y ") Then
                QueryString = "solve for Y " + str
            ElseIf str.ToLower.StartsWith("y =") And str.Contains(" x ") Then
                QueryString = "solve for x " + str
            ElseIf str.ToLower.StartsWith("y =") And str.Contains(" X ") Then
                QueryString = "solve for X " + str
            Else
                Dim s As String = InputBox(vbCrLf + "Please enter the variable to solve for", _
                                                    "", "", p.X + 52, p.Y + 10)
                If s = "" Then Return
                ' ----------------------------------------------------- test if exists and correct the case
                Dim s2 As String
                s2 = GetIdentifierSingleLetter_LowerCaseOrUpperCase(s)
                ' -----------------------------------------------------
                If s2 <> "" Then
                    QueryString = "solve for " + s2 + " " + str
                Else
                    MessageBox.Show("The variable """ + s + """ is not present in the given equation")
                End If
            End If
        Else
            QueryString = "simplify   " + str
        End If
    End Sub

    Friend Function Wolfram_IsValidQuery() As Boolean
        Return QueryString.ToLower.StartsWith("solve for") Or QueryString.ToLower.StartsWith("simplify")
    End Function


    Friend Sub Wolfram_EvaluateAndGetResults(ByVal tbox As RichTextBox)
        If Wolfram_IsValidQuery() Then
            Dim s As String = Strings.Left(QueryString, 11).ToUpper.Trim
            Dim oldcolor As Color = tbox.BackColor
            tbox.BackColor = Color.FromArgb(255, 230, 150)
            tbox.SetTextUsingUndo(vbCrLf + " Wolfram Alpha - " + s + _
                                  " - Please wait some seconds" + vbCrLf + " ")
            tbox.Refresh()
            Wolfram_StartEvaluateThread()
            '
            Wolfram_Response = ""
            Dim seconds As Single = 0
            Do
                Application.DoEvents()
                Threading.Thread.Sleep(100)
                seconds += 0.1F
                If seconds > 10 Then
                    tbox.Text = vbCrLf + " Waiting for max 60 seconds  ---  Elapsed seconds = " + _
                                          seconds.ToString("0") + vbCrLf + " (click to stop)"
                Else
                    tbox.Text += "."
                End If
                If seconds > 60 Then
                    Wolfram_Response = vbCrLf + " No response from Wolfram Alpha"
                End If
            Loop Until Wolfram_Response <> ""
            '
            tbox.SetTextUsingUndo(Wolfram_Response)
            tbox.BackColor = oldcolor
        End If
    End Sub

    Friend Sub Wolfram_StopWaiting()
        Wolfram_Response = vbCrLf + " Wait stopped by the user"
    End Sub

    ' ========================================================================== 
    '  Wolfram_Evaluate called with a thread
    ' ==========================================================================
    Private Wolfram_Response As String = ""
    Private WolframThread As Threading.Thread

    Friend Sub Wolfram_StartEvaluateThread()
        WolframThread = New Threading.Thread(AddressOf WolframEvaluateThread)
        WolframThread.IsBackground = True
        WolframThread.Start()
    End Sub

    Private Sub WolframEvaluateThread()
        Wolfram_Response = Wolfram_Evaluate()
    End Sub

    Friend Wolfram_ApiV2 As Boolean = True
    Friend Wolfram_Approximated As Boolean = False
    Friend Wolfram_MoreDigits As Boolean = False
    Friend Wolfram_WebAsHTML As Boolean = False
    Friend Wolfram_WebAsXML As Boolean = False

    Private Function Wolfram_Evaluate() As String
        '
        ' ------------------------------------------------- Compose and encode the URL
        Dim url As String
        QueryString = System.Web.HttpUtility.UrlEncode(QueryString)

        If Wolfram_ApiV2 Then
            url = "http://api.wolframalpha.com/v2/query.jsp?appid=" + AppId + "&input=" + QueryString
        Else
            url = "http://api.wolframalpha.com/v1/query.jsp?appid=" + AppId + "&input=" + QueryString
        End If
        '
        url += "&format=plaintext"
        '
        If Wolfram_Approximated Then
            url += "&podstate=Result__Approximate+form"
        Else
            url += "&podstate=Result__Exact+forms"
        End If
        If Wolfram_MoreDigits Then
            url += "&podstate=Result__More+digits"
        End If
        '
        ' ------------------------------------------------- use the URL to get the HTML
        If Wolfram_WebAsHTML Then
            Try
                Dim i As Int32 = url.IndexOf("&input=")
                url = url.Substring(i + 7)
                url = "http://www.wolframalpha.com/input/?i=" + url
                Process.Start(url)
                Return vbCrLf + " Opening the Wolfram Alpha web-site"
            Catch
            End Try
        End If
        '
        ' ------------------------------------------------- use the URL to get the XML
        If Wolfram_WebAsXML Then
            Process.Start(url)
            Return vbCrLf + " Opening the Wolfram Alpha web-site"
        End If
        '
        ' ------------------------------------------------- run the query
        Dim response As String = ""
        response = OpenSiteAndRead(url)
        '
        ' ------------------------------------------------- clean the response
        If response <> "" Then
            response = response.Replace("&gt;", ">")
            response = response.Replace("&lt;", "<")
            response = response.Replace("Ï€", "π")
            response = response.Replace("Â±", "±")
            response = response.Replace("â‰ˆ", "≈")
            response = RemoveTheFinalCrLf(response)
        Else
            response = vbCrLf + " No response from Wolfram Alpha"
        End If
        '
        Return response
    End Function

End Module

